'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var initializeFirebaseApp = require('initialize-firebase-app');
var getFirebaseRef = require('get-firebase-ref');
var sortBy = _interopDefault(require('lodash/sortby'));
var keys = _interopDefault(require('lodash/keys'));

const FirebaseContext = React.createContext({ firebase: {} });
function FirebaseProvider(props) {
    const { children, firebase } = props;
    initializeFirebaseApp.initializeFirebaseApp(props);
    return (React__default.createElement(FirebaseContext.Provider, { value: { firebase } }, children));
}
function useDatabase(firebaseQuery) {
    const { firebase } = React.useContext(FirebaseContext);
    const [isLoading, setIsLoading] = React.useState(false);
    const [value, setValue] = React.useState(null);
    const [error, setError] = React.useState(null);
    const [firebaseRef, setFirebaseRef] = React.useState({});
    function onValue(v) {
        const { keysOnly, isList, orderByChild, orderByKey, orderByValue } = firebaseQuery;
        if (!v)
            return;
        if (v.val() === null)
            return;
        const value = v.val();
        if (!keysOnly && !isList) {
            setValue(value);
            return;
        }
        const unsortedKeys = keys(value);
        const sortedKeys = orderByChild
            ? sortBy(unsortedKeys, key => value[key][orderByChild])
            : orderByKey
                ? sortBy(unsortedKeys, key => key)
                : orderByValue
                    ? sortBy(unsortedKeys, key => value[key])
                    : unsortedKeys;
        if (keysOnly) {
            setValue(sortedKeys);
            return;
        }
        if (isList) {
            const val = sortedKeys.map((key) => ({ key, data: value[key] }));
            setValue(val);
            return;
        }
        else {
            setValue(value);
            return;
        }
    }
    React.useEffect(() => {
        const ref = getFirebaseRef.getFirebaseRef(Object.assign({}, firebaseQuery, { firebase }));
        setFirebaseRef(firebaseRef);
        setIsLoading(true);
        if (firebaseQuery.once) {
            const unsub = ref.on("value", (v) => {
                setIsLoading(false);
                onValue(v);
            }, (err) => {
                setError(err);
            });
            return unsub;
        }
        ref.once("value", (v) => {
            setIsLoading(false);
            onValue(v);
        });
    });
    return { value, isLoading, ref: firebaseRef, error };
}

exports.FirebaseContext = FirebaseContext;
exports.FirebaseProvider = FirebaseProvider;
exports.useDatabase = useDatabase;
