import React, { useState, useEffect, useContext, createContext } from 'react';
import { initializeFirebaseApp } from 'initialize-firebase-app';
import { getFirebaseRef } from 'get-firebase-ref';
import sortBy from 'lodash/sortby';
import keys from 'lodash/keys';

const FirebaseContext = createContext({ firebase: {} });
function FirebaseProvider(props) {
    const { children, firebase } = props;
    initializeFirebaseApp(props);
    return (React.createElement(FirebaseContext.Provider, { value: { firebase } }, children));
}
function useDatabase(firebaseQuery) {
    const { firebase } = useContext(FirebaseContext);
    const [isLoading, setIsLoading] = useState(false);
    const [value, setValue] = useState(null);
    const [error, setError] = useState(null);
    const [firebaseRef, setFirebaseRef] = useState({});
    function onValue(v) {
        const { keysOnly, isList, orderByChild, orderByKey, orderByValue } = firebaseQuery;
        if (!v)
            return;
        if (v.val() === null)
            return;
        const value = v.val();
        if (!keysOnly && !isList) {
            setValue(value);
            return;
        }
        const unsortedKeys = keys(value);
        const sortedKeys = orderByChild
            ? sortBy(unsortedKeys, key => value[key][orderByChild])
            : orderByKey
                ? sortBy(unsortedKeys, key => key)
                : orderByValue
                    ? sortBy(unsortedKeys, key => value[key])
                    : unsortedKeys;
        if (keysOnly) {
            setValue(sortedKeys);
            return;
        }
        if (isList) {
            const val = sortedKeys.map((key) => ({ key, data: value[key] }));
            setValue(val);
            return;
        }
        else {
            setValue(value);
            return;
        }
    }
    useEffect(() => {
        const ref = getFirebaseRef(Object.assign({}, firebaseQuery, { firebase }));
        setFirebaseRef(firebaseRef);
        setIsLoading(true);
        if (firebaseQuery.once) {
            const unsub = ref.on("value", (v) => {
                setIsLoading(false);
                onValue(v);
            }, (err) => {
                setError(err);
            });
            return unsub;
        }
        ref.once("value", (v) => {
            setIsLoading(false);
            onValue(v);
        });
    });
    return { value, isLoading, ref: firebaseRef, error };
}

export { FirebaseContext, FirebaseProvider, useDatabase };
